#include "SerialPort.h"

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>

SerialPort* newSerialPort(char* device, int baud)
{
	SerialPort* serialPort = (SerialPort*)malloc(sizeof(SerialPort));
	
	serialPort->fileDescriptor = open(device, O_RDWR | O_NOCTTY | O_NONBLOCK);
	
	ioctl(serialPort->fileDescriptor, TIOCEXCL);
	fcntl(serialPort->fileDescriptor, F_SETFL, 0);
	
	tcgetattr(serialPort->fileDescriptor, &serialPort->originalAttributes);
	
	struct termios options = serialPort->originalAttributes;
	cfsetspeed(&options, baud);
	options.c_cflag |= CS8;
	tcsetattr(serialPort->fileDescriptor, TCSANOW, &options);
	
	return serialPort;
}

void deleteSerialPort(SerialPort* serialPort)
{
	if (serialPort == NULL)
		return;
		
	tcdrain(serialPort->fileDescriptor);
	
	tcsetattr(serialPort->fileDescriptor, TCSANOW, &serialPort->originalAttributes);
	
	close(serialPort->fileDescriptor);
	
	free(serialPort);
}

int spIsReadAvailable(SerialPort* serialPort)
{
	if (serialPort == NULL)
		return 0;
		
	fd_set readSet;
	struct timeval timeout;
	
	memset(&timeout, 0, sizeof(struct timeval));
	
	FD_ZERO(&readSet);
	FD_SET(serialPort->fileDescriptor, &readSet);
	
	int result = select(serialPort->fileDescriptor + 1, &readSet, NULL, NULL, &timeout);
	
	return result == 1;
}

int spRead(SerialPort* serialPort)
{
	if (serialPort == NULL)
		return -1;

	unsigned char result;
	
	read(serialPort->fileDescriptor, &result, 1);
	
	return result;
}

void spWrite(SerialPort* serialPort, int character)
{
	if (serialPort == NULL)
		return;
		
	unsigned char buffer = (unsigned char)character;
	
	write(serialPort->fileDescriptor, &buffer, 1);
}
