Object subclass: #CircularBuffer
	instanceVariableNames: 'byteArray available nextWriteIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimpleSerialPort'!

!CircularBuffer methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:48'!
available

	^ available! !

!CircularBuffer methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:46'!
initializeSize: anInteger

	byteArray _ ByteArray new: anInteger.
	available _ 0.
	nextWriteIndex _ 1.
! !

!CircularBuffer methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:58'!
next

	| nextReadIndex |
	available > 0 ifFalse: [self error: 'Buffer empty'].

	nextReadIndex _ nextWriteIndex - available.
	nextReadIndex < 1 ifTrue: [nextReadIndex _ nextReadIndex + byteArray size].

	available _ available - 1.

	^ byteArray at: nextReadIndex! !

!CircularBuffer methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:58'!
nextPut: aByte

	byteArray
		at: nextWriteIndex
		put: aByte.

	nextWriteIndex _ nextWriteIndex + 1.
	(nextWriteIndex > byteArray size) ifTrue: [nextWriteIndex _ 1].

	available _ (available + 1) min: byteArray size.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CircularBuffer class
	instanceVariableNames: ''!

!CircularBuffer class methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:45'!
new: anInteger

	^ self new initializeSize: anInteger! !


ExternalLibrary subclass: #SerialPortLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimpleSerialPort'!

!SerialPortLibrary methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:08'!
deleteSerialPort: aSerialPort

	<cdecl: void 'deleteSerialPort' (void*)>

	^ self externalCallFailed! !

!SerialPortLibrary methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:08'!
newSerialPort: device speed: baud

	<cdecl: void* 'newSerialPort' (char* long)>

	^ self externalCallFailed! !

!SerialPortLibrary methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:08'!
spIsReadAvailable: aSerialPort

	<cdecl: long 'spIsReadAvailable' (void*)>

	^ self externalCallFailed! !

!SerialPortLibrary methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:08'!
spRead: aSerialPort

	<cdecl: long 'spRead' (void*)>

	^ self externalCallFailed! !

!SerialPortLibrary methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:08'!
sp: aSerialPort write: aByte

	<cdecl: void 'spWrite' (void* long)>

	^ self externalCallFailed! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SerialPortLibrary class
	instanceVariableNames: ''!

!SerialPortLibrary class methodsFor: 'as yet unclassified' stamp: 'bvs 8/4/2006 14:47'!
moduleName

	^ 'libserialport.framework'! !


Object subclass: #SimpleSerialPort
	instanceVariableNames: 'portHandle readProcess buffer mutex'
	classVariableNames: 'Library'
	poolDictionaries: ''
	category: 'SimpleSerialPort'!

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 20:00'!
available

	^ buffer available! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 17:00'!
close

	readProcess terminate.
	self class library deleteSerialPort: portHandle.
	portHandle _ nil.
! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:54'!
initializeOnPort: aString speed: anInteger

	portHandle _ self class library
		newSerialPort: aString
		speed: anInteger.

	buffer _ CircularBuffer new: 1024.
	mutex _ Semaphore forMutualExclusion.

	readProcess _ [self readLoop] fork.
! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:55'!
loadByteIntoBuffer

	[(self class library spIsReadAvailable: portHandle) = 1] whileFalse:
		[(Delay forMilliseconds: 10) wait].

	mutex critical:
		[| byte |
		byte _ self class library spRead: portHandle.
		buffer nextPut: byte].
! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:55'!
readByte

	[buffer available > 0] whileFalse:
		[(Delay forMilliseconds: 10) wait].

	mutex critical: [^ buffer next].
! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:15'!
readCharacter

	^ self readByte asCharacter! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:37'!
readLoop

	[true] whileTrue: [self loadByteIntoBuffer].
! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:06'!
writeByte: anInteger

	self class library
		sp: portHandle
		write: anInteger
! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:05'!
writeCharacter: aCharacter

	self writeByte: aCharacter asciiValue! !

!SimpleSerialPort methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:12'!
writeString: aString

	aString do: [:each | self writeCharacter: each]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimpleSerialPort class
	instanceVariableNames: ''!

!SimpleSerialPort class methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 15:58'!
library

	^ Library ifNil: [Library _ SerialPortLibrary new]! !

!SimpleSerialPort class methodsFor: 'as yet unclassified' stamp: 'bvs 11/2/2006 16:04'!
newOnPort: aString speed: anInteger

	^ self new
		initializeOnPort: aString
		speed: anInteger! !
